import { RequestHandler } from 'express';
import { ProductService } from '@/services/product.service';
import { plainToInstance } from 'class-transformer';
import { ProductDto } from '@/dto/product.dto';

export class ProductController {
  private service: ProductService;

  constructor() {
    this.service = new ProductService();
  }

  /**
   * @swagger
   * /products:
   *    get:
   *      summary: Получить список продуктов
   *      tags:
   *      - Products
   *      responses:
   *        '200':
   *          description: Успешный запрос, получаем список продуктов
   *        '500':
   *          description: Внутренняя ошибка сервера
   */
  getProducts: RequestHandler = async (req, res): Promise<void> => {
    const products = await this.service.getProducts();
    res.send(products);
  };

  /**
   * @swagger
   * /products/{id}:
   *    get:
   *      summary: Получить один продукт
   *      tags:
   *      - Products
   *      parameters:
   *        - in: path
   *          name: id
   *          required: true
   *          description: ID продукта
   *          schema:
   *            type: integer
   *      responses:
   *        '200':
   *          description: Успешный запрос, получаем продукт
   *        '400':
   *          description: Неверный запрос, продукт с указанным ID не найден
   */
  getProduct: RequestHandler = async (req, res): Promise<void> => {
    try {
      const product = await this.service.getProduct(Number(req.params.id));
      res.send(product);
    } catch (e) {
      res.status(400).send({ message: (e as Error).message });
    }
  };


  /**
   * @swagger
   * /products:
   *    post:
   *      summary: Создать новый продукт
   *      tags:
   *      - Products
   *      requestBody:
   *        required: true
   *        content:
   *          multipart/form-data:
   *            schema:
   *              $ref: '#/components/schemas/ProductDto'
   *      responses:
   *        '200':
   *          description: Успешный запрос, создаем новый продукт
   *        '400':
   *          description: Ошибка в запросе, сообщение об ошибке
   *        '500':
   *          description: Внутренняя ошибка сервера
   */
  createProduct: RequestHandler = async (req, res): Promise<void> => {
    try {
      const productDto = plainToInstance(ProductDto, req.body);
      if (req.file) productDto.image = req.file.filename;
      const product = await this.service.createProduct(productDto);
      res.send(product);
    } catch (e) {
      if (Array.isArray(e)) {
        console.log(e);
        res.status(400).send(e);
      } else {
        res.status(500).send(e);
      }
    }
  };
}
